from typing import Any, Callable, Dict, Optional

import attr

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Node:
    """A logical node in the quantum processor's architecture.

    The existence of a node in the ISA `Architecture` does not necessarily mean that a given 1Q
    operation will be available on the node. This information is conveyed by the presence of the
    specific `node_id` in instances of `Instruction`."""

    node_id: int

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        node_id = self.node_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "node_id": node_id,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Node":
        d = src_dict.copy()
        node_id = d.pop("node_id")

        node = Node(
            node_id=node_id,
        )

        return node



from contextlib import asynccontextmanager, contextmanager
from typing import Optional

import httpx

from qcs_api_client.client._configuration import QCSClientConfiguration
from qcs_api_client.client.auth import QCSAuth, QCSAuthConfiguration


def _build_client_kwargs(
    *, configuration: Optional[QCSClientConfiguration] = None, kwarg_overrides: Optional[dict] = None
) -> dict:
    """
    Return kwargs used for construction of an httpx.BaseClient.
    """
    configuration = configuration or QCSClientConfiguration.load()
    auth_configuration = QCSAuthConfiguration(
        auth_server=configuration.auth_server,
    )
    auth = QCSAuth(
        client_configuration=configuration,
        auth_configuration=auth_configuration,
    )
    kwargs = dict(auth=auth, base_url=configuration.profile.api_url)

    if kwarg_overrides is not None:
        kwargs.update(kwarg_overrides)

    return kwargs


@contextmanager
def build_sync_client(
    *, configuration: Optional[QCSClientConfiguration] = None, client_kwargs: Optional[dict] = None
) -> httpx.Client:
    """
    Yield a client object suitable for use with the qcs_api_client.sync API functions.
    """
    with httpx.Client(**_build_client_kwargs(configuration=configuration, kwarg_overrides=client_kwargs)) as client:
        yield client


@asynccontextmanager
async def build_async_client(
    *, configuration: Optional[QCSClientConfiguration] = None, client_kwargs: Optional[dict] = None
) -> httpx.AsyncClient:
    """
    Yield a client object suitable for use with the qcs_api_client.asyncio API functions.
    """
    async with httpx.AsyncClient(
        **_build_client_kwargs(configuration=configuration, kwarg_overrides=client_kwargs)
    ) as client:
        yield client



from typing import Any, Callable, Dict, List, Optional

import attr

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class UserProfile:
    """  """

    email: str
    first_name: str
    last_name: str
    organization: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        email = self.email
        first_name = self.first_name
        last_name = self.last_name
        organization = self.organization

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "email": email,
                "firstName": first_name,
                "lastName": last_name,
                "organization": organization,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "UserProfile":
        d = src_dict.copy()
        email = d.pop("email")

        first_name = d.pop("firstName")

        last_name = d.pop("lastName")

        organization = d.pop("organization")

        user_profile = UserProfile(
            email=email,
            first_name=first_name,
            last_name=last_name,
            organization=organization,
        )

        user_profile.additional_properties = d
        return user_profile

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, Union, cast

import attr

from ..models.checksum_description import ChecksumDescription
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class ClientApplicationsDownloadLink:
    """  """

    url: str
    checksum_description: Union[ChecksumDescription, Unset] = UNSET
    platform: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        url = self.url
        checksum_description: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.checksum_description, Unset):
            checksum_description = self.checksum_description.to_dict()

        platform = self.platform

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "url": url,
            }
        )
        if checksum_description is not UNSET:
            field_dict["checksumDescription"] = checksum_description
        if platform is not UNSET:
            field_dict["platform"] = platform

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "ClientApplicationsDownloadLink":
        d = src_dict.copy()
        url = d.pop("url")

        checksum_description: Union[ChecksumDescription, Unset] = UNSET
        _checksum_description = d.pop("checksumDescription", UNSET)
        if _checksum_description is not None and not isinstance(_checksum_description, Unset):
            checksum_description = ChecksumDescription.from_dict(cast(Dict[str, Any], _checksum_description))

        platform = d.pop("platform", UNSET)

        client_applications_download_link = ClientApplicationsDownloadLink(
            url=url,
            checksum_description=checksum_description,
            platform=platform,
        )

        client_applications_download_link.additional_properties = d
        return client_applications_download_link

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, cast

import attr

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Edge:
    """A degree-two logical connection in the quantum processor's architecture.

    The existence of an edge in the ISA `Architecture` does not necessarily mean that a given 2Q
    operation will be available on the edge. This information is conveyed by the presence of the
    two `node_id` values in instances of `Instruction`.

    Note that edges are undirected in this model. Thus edge :math:`(a, b)` is equivalent to edge
    :math:`(b, a)`."""

    node_ids: List[int]

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        node_ids = self.node_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "node_ids": node_ids,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Edge":
        d = src_dict.copy()
        node_ids = cast(List[int], d.pop("node_ids"))

        edge = Edge(
            node_ids=node_ids,
        )

        return edge



import datetime
from typing import Any, Callable, Dict, List, Optional

import attr
from dateutil.parser import isoparse
from rfc3339 import rfc3339

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Group:
    """  """

    created_time: datetime.datetime
    description: str
    id: str
    last_membership_updated_time: datetime.datetime
    name: str
    updated_time: datetime.datetime
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        assert self.created_time.tzinfo is not None, "Datetime must have timezone information"
        created_time = rfc3339(self.created_time)

        description = self.description
        id = self.id
        assert self.last_membership_updated_time.tzinfo is not None, "Datetime must have timezone information"
        last_membership_updated_time = rfc3339(self.last_membership_updated_time)

        name = self.name
        assert self.updated_time.tzinfo is not None, "Datetime must have timezone information"
        updated_time = rfc3339(self.updated_time)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdTime": created_time,
                "description": description,
                "id": id,
                "lastMembershipUpdatedTime": last_membership_updated_time,
                "name": name,
                "updatedTime": updated_time,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Group":
        d = src_dict.copy()
        created_time = isoparse(d.pop("createdTime"))

        description = d.pop("description")

        id = d.pop("id")

        last_membership_updated_time = isoparse(d.pop("lastMembershipUpdatedTime"))

        name = d.pop("name")

        updated_time = isoparse(d.pop("updatedTime"))

        group = Group(
            created_time=created_time,
            description=description,
            id=id,
            last_membership_updated_time=last_membership_updated_time,
            name=name,
            updated_time=updated_time,
        )

        group.additional_properties = d
        return group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional

import attr

from ..models.edge import Edge
from ..models.family import Family
from ..models.node import Node
from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Architecture:
    """Represents the logical underlying architecture of a quantum processor.

    The architecture is defined in detail by the nodes and edges that constitute the quantum
    processor. This defines the set of all nodes that could be operated upon, and indicates to
    some approximation their physical layout. The main purpose of this is to support geometry
    calculations that are independent of the available operations, and rendering ISA-based
    information. Architecture layouts are defined by the `family`, as follows.

    The "Aspen" family of quantum processor indicates a 2D planar grid layout of octagon unit
    cells. The `node_id` in this architecture is computed as :math:`100 p_y + 10 p_x + p_u` where
    :math:`p_y` is the zero-based Y position in the unit cell grid, :math:`p_x` is the zero-based
    X position in the unit cell grid, and :math:`p_u` is the zero-based position in the octagon
    unit cell and always ranges from 0 to 7. This scheme has a natural size limit of a 10x10
    unit cell grid, which permits the architecture to scale up to 800 nodes.

    Note that the operations that are actually available are defined entirely by `Operation`
    instances. The presence of a node or edge in the `Architecture` model provides no guarantee
    that any 1Q or 2Q operation will be available to users writing QUIL programs."""

    edges: List[Edge]
    family: Family
    nodes: List[Node]

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        edges = []
        for edges_item_data in self.edges:
            edges_item = edges_item_data.to_dict()

            edges.append(edges_item)

        family = self.family.value

        nodes = []
        for nodes_item_data in self.nodes:
            nodes_item = nodes_item_data.to_dict()

            nodes.append(nodes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "edges": edges,
                "family": family,
                "nodes": nodes,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Architecture":
        d = src_dict.copy()
        edges = []
        _edges = d.pop("edges")
        for edges_item_data in _edges:
            edges_item = Edge.from_dict(edges_item_data)

            edges.append(edges_item)

        family = Family(d.pop("family"))

        nodes = []
        _nodes = d.pop("nodes")
        for nodes_item_data in _nodes:
            nodes_item = Node.from_dict(nodes_item_data)

            nodes.append(nodes_item)

        architecture = Architecture(
            edges=edges,
            family=family,
            nodes=nodes,
        )

        return architecture



from qcs_api_client.client.auth import QCSAuth, QCSAuthConfiguration, QCSAuthRefreshError
from qcs_api_client.client.client import build_sync_client, build_async_client
from qcs_api_client.client._configuration import *



from httpx import Response, HTTPStatusError
from http import HTTPStatus
from typing import cast, Dict, Any, Optional
from json import JSONDecodeError

from ..models.error import Error


class QCSHTTPStatusError(HTTPStatusError):
    def __init__(self, message: str, *, response: Response, error: Optional[Error]) -> None:
        super().__init__(message, request=response.request, response=response)
        self.error = error


def raise_for_status(res: Response):
    """
    Raise the `QCSHTTPStatusError` if one occurred.
    """
    if res.request is None:
        raise RuntimeError(
            "Cannot call `raise_for_status` as the request " "instance has not been set on this response."
        )
    elif res.status_code < HTTPStatus.BAD_REQUEST:
        return None

    message = f"QCS API call {res.request.method} {res.request.url} failed with status {res.status_code}: {res.text}"
    error = None
    try:
        error = Error.from_dict(cast(Dict[str, Any], res.json()))
    except (JSONDecodeError, KeyError):
        pass

    raise QCSHTTPStatusError(message, error=error, response=res)



from typing import Any, Callable, Dict, List, Optional, Union, cast

import attr

from ..models.translate_native_quil_to_encrypted_binary_response_memory_descriptors import (
    TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors,
)
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class TranslateNativeQuilToEncryptedBinaryResponse:
    """  """

    program: str
    memory_descriptors: Union[TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors, Unset] = UNSET
    ro_sources: Union[Unset, List[List[str]]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        program = self.program
        memory_descriptors: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.memory_descriptors, Unset):
            memory_descriptors = self.memory_descriptors.to_dict()

        ro_sources: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.ro_sources, Unset):
            ro_sources = []
            for ro_sources_item_data in self.ro_sources:
                ro_sources_item = ro_sources_item_data

                ro_sources.append(ro_sources_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "program": program,
            }
        )
        if memory_descriptors is not UNSET:
            field_dict["memoryDescriptors"] = memory_descriptors
        if ro_sources is not UNSET:
            field_dict["roSources"] = ro_sources

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "TranslateNativeQuilToEncryptedBinaryResponse":
        d = src_dict.copy()
        program = d.pop("program")

        memory_descriptors: Union[TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors, Unset] = UNSET
        _memory_descriptors = d.pop("memoryDescriptors", UNSET)
        if _memory_descriptors is not None and not isinstance(_memory_descriptors, Unset):
            memory_descriptors = TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors.from_dict(
                cast(Dict[str, Any], _memory_descriptors)
            )

        ro_sources = []
        _ro_sources = d.pop("roSources", UNSET)
        for ro_sources_item_data in _ro_sources or []:
            ro_sources_item = cast(List[str], ro_sources_item_data)

            ro_sources.append(ro_sources_item)

        translate_native_quil_to_encrypted_binary_response = TranslateNativeQuilToEncryptedBinaryResponse(
            program=program,
            memory_descriptors=memory_descriptors,
            ro_sources=ro_sources,
        )

        translate_native_quil_to_encrypted_binary_response.additional_properties = d
        return translate_native_quil_to_encrypted_binary_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict

import httpx
from retrying import retry

from ...models.health import Health
from ...types import Response
from ...util.errors import QCSHTTPStatusError, raise_for_status
from ...util.retry import DEFAULT_RETRY_ARGUMENTS


def _get_kwargs() -> Dict[str, Any]:

    return {}


def _parse_response(*, response: httpx.Response) -> Health:
    raise_for_status(response)
    if response.status_code == 200:
        response_200 = Health.from_dict(response.json())

        return response_200
    else:
        raise QCSHTTPStatusError(
            f"Unexpected response: status code {response.status_code}", response=response, error=None
        )


def _build_response(*, response: httpx.Response) -> Response[Health]:
    """
    Construct the Response class from the raw ``httpx.Response``.
    """
    return Response.build_from_httpx_response(response=response, parse_function=_parse_response)


@retry(**DEFAULT_RETRY_ARGUMENTS)
def sync(
    *,
    client: httpx.Client,
    httpx_request_kwargs: Dict[str, Any] = {},
) -> Response[Health]:
    url = "/"

    kwargs = _get_kwargs()
    kwargs.update(httpx_request_kwargs)
    response = client.request(
        "get",
        url,
        **kwargs,
    )
    return _build_response(response=response)


@retry(**DEFAULT_RETRY_ARGUMENTS)
def sync_from_dict(
    *,
    client: httpx.Client,
    httpx_request_kwargs: Dict[str, Any] = {},
) -> Response[Health]:

    url = "/"

    kwargs = _get_kwargs()
    kwargs.update(httpx_request_kwargs)
    response = client.request(
        "get",
        url,
        **kwargs,
    )
    return _build_response(response=response)


@retry(**DEFAULT_RETRY_ARGUMENTS)
async def asyncio(
    *,
    client: httpx.AsyncClient,
    httpx_request_kwargs: Dict[str, Any] = {},
) -> Response[Health]:
    url = "/"

    kwargs = _get_kwargs()
    kwargs.update(httpx_request_kwargs)
    response = await client.request(
        "get",
        url,
        **kwargs,
    )
    return _build_response(response=response)


@retry(**DEFAULT_RETRY_ARGUMENTS)
async def asyncio_from_dict(
    *,
    client: httpx.AsyncClient,
    httpx_request_kwargs: Dict[str, Any] = {},
) -> Response[Health]:

    url = "/"

    kwargs = _get_kwargs()
    kwargs.update(httpx_request_kwargs)
    response = await client.request(
        "get",
        url,
        **kwargs,
    )
    return _build_response(response=response)



from qcs_api_client.api.account.add_group_user import asyncio_from_dict as add_group_user
from qcs_api_client.api.account.list_group_users import asyncio_from_dict as list_group_users
from qcs_api_client.api.account.list_user_groups import asyncio_from_dict as list_user_groups
from qcs_api_client.api.account.remove_group_user import asyncio_from_dict as remove_group_user
from qcs_api_client.api.authentication.auth_email_password_reset_token import (
    asyncio_from_dict as auth_email_password_reset_token,
)
from qcs_api_client.api.authentication.auth_get_user import asyncio_from_dict as auth_get_user
from qcs_api_client.api.authentication.auth_reset_password import asyncio_from_dict as auth_reset_password
from qcs_api_client.api.authentication.auth_reset_password_with_token import (
    asyncio_from_dict as auth_reset_password_with_token,
)
from qcs_api_client.api.client_applications.check_client_application import (
    asyncio_from_dict as check_client_application,
)
from qcs_api_client.api.client_applications.get_client_application import asyncio_from_dict as get_client_application
from qcs_api_client.api.client_applications.list_client_applications import (
    asyncio_from_dict as list_client_applications,
)
from qcs_api_client.api.default.get_health import asyncio_from_dict as get_health
from qcs_api_client.api.default.health_check import asyncio_from_dict as health_check
from qcs_api_client.api.endpoints.create_endpoint import asyncio_from_dict as create_endpoint
from qcs_api_client.api.endpoints.delete_endpoint import asyncio_from_dict as delete_endpoint
from qcs_api_client.api.endpoints.get_default_endpoint import asyncio_from_dict as get_default_endpoint
from qcs_api_client.api.endpoints.get_endpoint import asyncio_from_dict as get_endpoint
from qcs_api_client.api.endpoints.list_endpoints import asyncio_from_dict as list_endpoints
from qcs_api_client.api.engagements.create_engagement import asyncio_from_dict as create_engagement
from qcs_api_client.api.quantum_processors.get_instruction_set_architecture import (
    asyncio_from_dict as get_instruction_set_architecture,
)
from qcs_api_client.api.quantum_processors.get_quantum_processor import asyncio_from_dict as get_quantum_processor
from qcs_api_client.api.quantum_processors.list_quantum_processors import asyncio_from_dict as list_quantum_processors
from qcs_api_client.api.reservations.create_reservation import asyncio_from_dict as create_reservation
from qcs_api_client.api.reservations.delete_reservation import asyncio_from_dict as delete_reservation
from qcs_api_client.api.reservations.find_available_reservations import asyncio_from_dict as find_available_reservations
from qcs_api_client.api.reservations.list_reservations import asyncio_from_dict as list_reservations
from qcs_api_client.api.translation.get_quilt_calibrations import asyncio_from_dict as get_quilt_calibrations
from qcs_api_client.api.translation.translate_native_quil_to_encrypted_binary import (
    asyncio_from_dict as translate_native_quil_to_encrypted_binary,
)



import datetime
from typing import Any, Callable, Dict, List, Optional, Union

import attr
from dateutil.parser import isoparse
from rfc3339 import rfc3339

from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class CreateReservationRequest:
    """  """

    end_time: datetime.datetime
    quantum_processor_id: str
    start_time: datetime.datetime
    notes: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        assert self.end_time.tzinfo is not None, "Datetime must have timezone information"
        end_time = rfc3339(self.end_time)

        quantum_processor_id = self.quantum_processor_id
        assert self.start_time.tzinfo is not None, "Datetime must have timezone information"
        start_time = rfc3339(self.start_time)

        notes = self.notes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "endTime": end_time,
                "quantumProcessorId": quantum_processor_id,
                "startTime": start_time,
            }
        )
        if notes is not UNSET:
            field_dict["notes"] = notes

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "CreateReservationRequest":
        d = src_dict.copy()
        end_time = isoparse(d.pop("endTime"))

        quantum_processor_id = d.pop("quantumProcessorId")

        start_time = isoparse(d.pop("startTime"))

        notes = d.pop("notes", UNSET)

        create_reservation_request = CreateReservationRequest(
            end_time=end_time,
            quantum_processor_id=quantum_processor_id,
            start_time=start_time,
            notes=notes,
        )

        create_reservation_request.additional_properties = d
        return create_reservation_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class InviteUserRequest:
    """  """

    email: str
    group_name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        email = self.email
        group_name = self.group_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "email": email,
            }
        )
        if group_name is not UNSET:
            field_dict["groupName"] = group_name

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "InviteUserRequest":
        d = src_dict.copy()
        email = d.pop("email")

        group_name = d.pop("groupName", UNSET)

        invite_user_request = InviteUserRequest(
            email=email,
            group_name=group_name,
        )

        invite_user_request.additional_properties = d
        return invite_user_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..models.client_applications_download_link import ClientApplicationsDownloadLink
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class ClientApplication:
    """  """

    latest_version: str
    name: str
    supported: bool
    details_uri: Union[Unset, str] = UNSET
    links: Union[Unset, List[ClientApplicationsDownloadLink]] = UNSET
    minimum_version: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        latest_version = self.latest_version
        name = self.name
        supported = self.supported
        details_uri = self.details_uri
        links: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.links, Unset):
            links = []
            for links_item_data in self.links:
                links_item = links_item_data.to_dict()

                links.append(links_item)

        minimum_version = self.minimum_version

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "latestVersion": latest_version,
                "name": name,
                "supported": supported,
            }
        )
        if details_uri is not UNSET:
            field_dict["detailsUri"] = details_uri
        if links is not UNSET:
            field_dict["links"] = links
        if minimum_version is not UNSET:
            field_dict["minimumVersion"] = minimum_version

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "ClientApplication":
        d = src_dict.copy()
        latest_version = d.pop("latestVersion")

        name = d.pop("name")

        supported = d.pop("supported")

        details_uri = d.pop("detailsUri", UNSET)

        links = []
        _links = d.pop("links", UNSET)
        for links_item_data in _links or []:
            links_item = ClientApplicationsDownloadLink.from_dict(links_item_data)

            links.append(links_item)

        minimum_version = d.pop("minimumVersion", UNSET)

        client_application = ClientApplication(
            latest_version=latest_version,
            name=name,
            supported=supported,
            details_uri=details_uri,
            links=links,
            minimum_version=minimum_version,
        )

        client_application.additional_properties = d
        return client_application

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from qcs_api_client.util.errors import QCSHTTPStatusError


DEFAULT_RETRY_STATUS_CODES = {
    HTTPStatus.BAD_GATEWAY,
    HTTPStatus.SERVICE_UNAVAILABLE,
    HTTPStatus.GATEWAY_TIMEOUT,
}


def _is_exception_retryable(exception):
    if isinstance(exception, QCSHTTPStatusError):
        return exception.response.status_code in DEFAULT_RETRY_STATUS_CODES
    return False


DEFAULT_RETRY_ARGUMENTS = {
    "stop_max_attempt_number": 3,
    "wait_exponential_multiplier": 200,
    "wait_exponential_max": 1000,
    "retry_on_exception": _is_exception_retryable,
}



from enum import Enum


class ValidationErrorIn(str, Enum):
    HEADER = "header"
    QUERY = "query"
    PATH = "path"
    BODY = "body"

    def __str__(self) -> str:
        return str(self.value)



import os
from typing import Any, Dict

from pydantic import BaseModel
from pydantic.utils import deep_update


class _EnvironmentBaseModel(BaseModel):
    """
    A utility class meant for inheritance by classes which initialize using environment variables.

    Ensure that ``env_prefix`` is set on the metaclass ``Config``.
    """

    def _read_env(self) -> Dict[str, str]:
        """
        Read the value of each primitive-typed field from the environment.

        Environment variable name is case insensitive. The prefix is set as ``env_prefix``on the
        model's ``Config``.
        """
        if not getattr(self.__config__, "env_prefix", None):
            return {}

        data: Dict[str, str] = {}
        env: Dict[str, str] = {k.lower(): v for k, v in os.environ.items()}

        for field in self.__fields__.values():

            # We only read primitives from the environment, no JSON parsing
            if not field.is_complex():
                env_name = (self.__config__.env_prefix + field.name).lower()
                if env_name in env:
                    data[field.name] = env[env_name]

        return data


class EnvironmentModel(_EnvironmentBaseModel):
    """
    A BaseModel which also attempts to read primitive field values from the environment on
    initialization.
    """

    def __init__(self, **kwargs: Any) -> None:
        """
        Load values in this order of precedence:
        1. Runtime keyword arguments
        2. Environment variables
        3. Field default values
        """
        values = deep_update(
            self._read_env(),
            kwargs,
        )

        super().__init__(**values)



import datetime
from typing import Any, Callable, Dict, List, Optional, Union, cast

import attr
from dateutil.parser import isoparse
from rfc3339 import rfc3339

from ..models.user_profile import UserProfile
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class User:
    """  """

    created_time: datetime.datetime
    id: int
    idp_id: str
    profile: Union[UserProfile, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        assert self.created_time.tzinfo is not None, "Datetime must have timezone information"
        created_time = rfc3339(self.created_time)

        id = self.id
        idp_id = self.idp_id
        profile: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile, Unset):
            profile = self.profile.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdTime": created_time,
                "id": id,
                "idpId": idp_id,
            }
        )
        if profile is not UNSET:
            field_dict["profile"] = profile

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "User":
        d = src_dict.copy()
        created_time = isoparse(d.pop("createdTime"))

        id = d.pop("id")

        idp_id = d.pop("idpId")

        profile: Union[UserProfile, Unset] = UNSET
        _profile = d.pop("profile", UNSET)
        if _profile is not None and not isinstance(_profile, Unset):
            profile = UserProfile.from_dict(cast(Dict[str, Any], _profile))

        user = User(
            created_time=created_time,
            id=id,
            idp_id=idp_id,
            profile=profile,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..models.characteristic import Characteristic
from ..models.operation_site import OperationSite
from ..models.parameter import Parameter
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Operation:
    """ An operation, with its sites and site-independent characteristics. """

    characteristics: List[Characteristic]
    name: str
    parameters: List[Parameter]
    sites: List[OperationSite]
    node_count: Union[Unset, int] = UNSET

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        characteristics = []
        for characteristics_item_data in self.characteristics:
            characteristics_item = characteristics_item_data.to_dict()

            characteristics.append(characteristics_item)

        name = self.name
        parameters = []
        for parameters_item_data in self.parameters:
            parameters_item = parameters_item_data.to_dict()

            parameters.append(parameters_item)

        sites = []
        for sites_item_data in self.sites:
            sites_item = sites_item_data.to_dict()

            sites.append(sites_item)

        node_count = self.node_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "characteristics": characteristics,
                "name": name,
                "parameters": parameters,
                "sites": sites,
            }
        )
        if node_count is not UNSET:
            field_dict["node_count"] = node_count

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Operation":
        d = src_dict.copy()
        characteristics = []
        _characteristics = d.pop("characteristics")
        for characteristics_item_data in _characteristics:
            characteristics_item = Characteristic.from_dict(characteristics_item_data)

            characteristics.append(characteristics_item)

        name = d.pop("name")

        parameters = []
        _parameters = d.pop("parameters")
        for parameters_item_data in _parameters:
            parameters_item = Parameter.from_dict(parameters_item_data)

            parameters.append(parameters_item)

        sites = []
        _sites = d.pop("sites")
        for sites_item_data in _sites:
            sites_item = OperationSite.from_dict(sites_item_data)

            sites.append(sites_item)

        node_count = d.pop("node_count", UNSET)

        operation = Operation(
            characteristics=characteristics,
            name=name,
            parameters=parameters,
            sites=sites,
            node_count=node_count,
        )

        return operation



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..models.quantum_processor import QuantumProcessor
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class ListQuantumProcessorsResponse:
    """  """

    quantum_processors: List[QuantumProcessor]
    next_page_token: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        quantum_processors = []
        for quantum_processors_item_data in self.quantum_processors:
            quantum_processors_item = quantum_processors_item_data.to_dict()

            quantum_processors.append(quantum_processors_item)

        next_page_token = self.next_page_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "quantumProcessors": quantum_processors,
            }
        )
        if next_page_token is not UNSET:
            field_dict["nextPageToken"] = next_page_token

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "ListQuantumProcessorsResponse":
        d = src_dict.copy()
        quantum_processors = []
        _quantum_processors = d.pop("quantumProcessors")
        for quantum_processors_item_data in _quantum_processors:
            quantum_processors_item = QuantumProcessor.from_dict(quantum_processors_item_data)

            quantum_processors.append(quantum_processors_item)

        next_page_token = d.pop("nextPageToken", UNSET)

        list_quantum_processors_response = ListQuantumProcessorsResponse(
            quantum_processors=quantum_processors,
            next_page_token=next_page_token,
        )

        list_quantum_processors_response.additional_properties = d
        return list_quantum_processors_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional

import attr

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class EngagementCredentials:
    """Credentials are the ZeroMQ CURVE Keys used to encrypt the connection with the Quantum Processor
    Endpoint."""

    client_public: str
    client_secret: str
    server_public: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        client_public = self.client_public
        client_secret = self.client_secret
        server_public = self.server_public

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "clientPublic": client_public,
                "clientSecret": client_secret,
                "serverPublic": server_public,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "EngagementCredentials":
        d = src_dict.copy()
        client_public = d.pop("clientPublic")

        client_secret = d.pop("clientSecret")

        server_public = d.pop("serverPublic")

        engagement_credentials = EngagementCredentials(
            client_public=client_public,
            client_secret=client_secret,
            server_public=server_public,
        )

        engagement_credentials.additional_properties = d
        return engagement_credentials

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional

import attr

from ..models.architecture import Architecture
from ..models.operation import Operation
from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class InstructionSetArchitecture:
    """The native instruction set architecture of a quantum processor, annotated with characteristics.

    The operations described by the `instructions` field are named by their QUIL instruction name,
    while the operation described by the `benchmarks` field are named by their benchmark routine
    and are a future extension point that will be populated in future iterations.

    The characteristics that annotate both instructions and benchmarks assist the user to generate
    the best native QUIL program for a desired task, and so are provided as part of the native ISA."""

    architecture: Architecture
    benchmarks: List[Operation]
    instructions: List[Operation]
    name: str

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        architecture = self.architecture.to_dict()

        benchmarks = []
        for benchmarks_item_data in self.benchmarks:
            benchmarks_item = benchmarks_item_data.to_dict()

            benchmarks.append(benchmarks_item)

        instructions = []
        for instructions_item_data in self.instructions:
            instructions_item = instructions_item_data.to_dict()

            instructions.append(instructions_item)

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "architecture": architecture,
                "benchmarks": benchmarks,
                "instructions": instructions,
                "name": name,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "InstructionSetArchitecture":
        d = src_dict.copy()
        architecture = Architecture.from_dict(d.pop("architecture"))

        benchmarks = []
        _benchmarks = d.pop("benchmarks")
        for benchmarks_item_data in _benchmarks:
            benchmarks_item = Operation.from_dict(benchmarks_item_data)

            benchmarks.append(benchmarks_item)

        instructions = []
        _instructions = d.pop("instructions")
        for instructions_item_data in _instructions:
            instructions_item = Operation.from_dict(instructions_item_data)

            instructions.append(instructions_item)

        name = d.pop("name")

        instruction_set_architecture = InstructionSetArchitecture(
            architecture=architecture,
            benchmarks=benchmarks,
            instructions=instructions,
            name=name,
        )

        return instruction_set_architecture



from typing import Any, Callable, Dict, List, Optional, cast

import attr

from ..models.characteristic import Characteristic
from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class OperationSite:
    """ A site for an operation, with its site-dependent characteristics. """

    characteristics: List[Characteristic]
    node_ids: List[int]

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        characteristics = []
        for characteristics_item_data in self.characteristics:
            characteristics_item = characteristics_item_data.to_dict()

            characteristics.append(characteristics_item)

        node_ids = self.node_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "characteristics": characteristics,
                "node_ids": node_ids,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "OperationSite":
        d = src_dict.copy()
        characteristics = []
        _characteristics = d.pop("characteristics")
        for characteristics_item_data in _characteristics:
            characteristics_item = Characteristic.from_dict(characteristics_item_data)

            characteristics.append(characteristics_item)

        node_ids = cast(List[int], d.pop("node_ids"))

        operation_site = OperationSite(
            characteristics=characteristics,
            node_ids=node_ids,
        )

        return operation_site



from typing import Any, Callable, Dict, List, Optional

import attr

from ..models.endpoint_addresses import EndpointAddresses
from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Endpoint:
    """An Endpoint is the entry point for remote access to a QuantumProcessor."""

    address: str
    addresses: EndpointAddresses
    healthy: bool
    id: str
    mock: bool
    quantum_processor_id: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        address = self.address
        addresses = self.addresses.to_dict()

        healthy = self.healthy
        id = self.id
        mock = self.mock
        quantum_processor_id = self.quantum_processor_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "address": address,
                "addresses": addresses,
                "healthy": healthy,
                "id": id,
                "mock": mock,
                "quantumProcessorId": quantum_processor_id,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Endpoint":
        d = src_dict.copy()
        address = d.pop("address")

        addresses = EndpointAddresses.from_dict(d.pop("addresses"))

        healthy = d.pop("healthy")

        id = d.pop("id")

        mock = d.pop("mock")

        quantum_processor_id = d.pop("quantumProcessorId")

        endpoint = Endpoint(
            address=address,
            addresses=addresses,
            healthy=healthy,
            id=id,
            mock=mock,
            quantum_processor_id=quantum_processor_id,
        )

        endpoint.additional_properties = d
        return endpoint

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class RemoveGroupUserRequest:
    """ Must provide either `userId` or `userEmail` and `groupId` or `groupName`. """

    group_id: Union[Unset, str] = UNSET
    group_name: Union[Unset, str] = UNSET
    user_email: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        group_id = self.group_id
        group_name = self.group_name
        user_email = self.user_email
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if group_id is not UNSET:
            field_dict["groupId"] = group_id
        if group_name is not UNSET:
            field_dict["groupName"] = group_name
        if user_email is not UNSET:
            field_dict["userEmail"] = user_email
        if user_id is not UNSET:
            field_dict["userId"] = user_id

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "RemoveGroupUserRequest":
        d = src_dict.copy()
        group_id = d.pop("groupId", UNSET)

        group_name = d.pop("groupName", UNSET)

        user_email = d.pop("userEmail", UNSET)

        user_id = d.pop("userId", UNSET)

        remove_group_user_request = RemoveGroupUserRequest(
            group_id=group_id,
            group_name=group_name,
            user_email=user_email,
            user_id=user_id,
        )

        remove_group_user_request.additional_properties = d
        return remove_group_user_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class CreateEngagementRequest:
    """  """

    endpoint_id: Union[Unset, str] = UNSET
    quantum_processor_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        endpoint_id = self.endpoint_id
        quantum_processor_id = self.quantum_processor_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if endpoint_id is not UNSET:
            field_dict["endpointId"] = endpoint_id
        if quantum_processor_id is not UNSET:
            field_dict["quantumProcessorId"] = quantum_processor_id

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "CreateEngagementRequest":
        d = src_dict.copy()
        endpoint_id = d.pop("endpointId", UNSET)

        quantum_processor_id = d.pop("quantumProcessorId", UNSET)

        create_engagement_request = CreateEngagementRequest(
            endpoint_id=endpoint_id,
            quantum_processor_id=quantum_processor_id,
        )

        create_engagement_request.additional_properties = d
        return create_engagement_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..models.validation_error import ValidationError
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Error:
    """  """

    code: str
    message: str
    request_id: str
    validation_errors: Union[Unset, List[ValidationError]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        code = self.code
        message = self.message
        request_id = self.request_id
        validation_errors: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.validation_errors, Unset):
            validation_errors = []
            for validation_errors_item_data in self.validation_errors:
                validation_errors_item = validation_errors_item_data.to_dict()

                validation_errors.append(validation_errors_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "code": code,
                "message": message,
                "requestId": request_id,
            }
        )
        if validation_errors is not UNSET:
            field_dict["validationErrors"] = validation_errors

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Error":
        d = src_dict.copy()
        code = d.pop("code")

        message = d.pop("message")

        request_id = d.pop("requestId")

        validation_errors = []
        _validation_errors = d.pop("validationErrors", UNSET)
        for validation_errors_item_data in _validation_errors or []:
            validation_errors_item = ValidationError.from_dict(validation_errors_item_data)

            validation_errors.append(validation_errors_item)

        error = Error(
            code=code,
            message=message,
            request_id=request_id,
            validation_errors=validation_errors,
        )

        error.additional_properties = d
        return error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional

import attr

from ..models.parameter_spec import ParameterSpec
from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors:
    """  """

    additional_properties: Dict[str, ParameterSpec] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        field_dict.update({})

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors":
        d = src_dict.copy()
        translate_native_quil_to_encrypted_binary_response_memory_descriptors = (
            TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors()
        )

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = ParameterSpec.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        translate_native_quil_to_encrypted_binary_response_memory_descriptors.additional_properties = (
            additional_properties
        )
        return translate_native_quil_to_encrypted_binary_response_memory_descriptors

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> ParameterSpec:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: ParameterSpec) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional

import attr

from ..models.reservation import Reservation
from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class ListReservationsResponse:
    """  """

    next_page_token: str
    reservations: List[Reservation]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        next_page_token = self.next_page_token
        reservations = []
        for reservations_item_data in self.reservations:
            reservations_item = reservations_item_data.to_dict()

            reservations.append(reservations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "nextPageToken": next_page_token,
                "reservations": reservations,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "ListReservationsResponse":
        d = src_dict.copy()
        next_page_token = d.pop("nextPageToken")

        reservations = []
        _reservations = d.pop("reservations")
        for reservations_item_data in _reservations:
            reservations_item = Reservation.from_dict(reservations_item_data)

            reservations.append(reservations_item)

        list_reservations_response = ListReservationsResponse(
            next_page_token=next_page_token,
            reservations=reservations,
        )

        list_reservations_response.additional_properties = d
        return list_reservations_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict
from pydantic import BaseModel
from pydantic.fields import Field
from pydantic.networks import HttpUrl
from qcs_api_client.client._configuration.environment import EnvironmentModel
from qcs_api_client.client._configuration.file import QCSClientConfigurationFile


class QCSAuthServer(BaseModel):
    """Specifies an OAuth2 authorization server against which to refresh tokens."""

    client_id: str
    issuer: str

    def authorize_url(self):
        return f"{self.issuer}/v1/authorize"

    def token_url(self):
        return f"{self.issuer}/v1/token"

    @staticmethod
    def scopes():
        return ["offline_access"]

    class Config:
        env_prefix = "QCS_SETTINGS_AUTH_SERVER_"


_DEFAULT_AUTH_SERVER = QCSAuthServer(
    client_id="0oa3ykoirzDKpkfzk357",
    issuer="https://auth.qcs.rigetti.com/oauth2/aus8jcovzG0gW2TUG355",
)


class QCSClientConfigurationSettingsApplicationsCLI(EnvironmentModel):
    verbosity: str = ""

    class Config:
        env_prefix = "QCS_SETTINGS_APPLICATIONS_CLI_"


class QCSClientConfigurationSettingsApplicationsPyquil(EnvironmentModel):
    qvm_url: str = "http://127.0.0.1:5000"
    quilc_url: str = "tcp://127.0.0.1:5555"

    class Config:
        env_prefix = "QCS_SETTINGS_APPLICATIONS_PYQUIL_"


class QCSClientConfigurationSettingsApplications(BaseModel):
    """Section of a profile specifying per-application settings."""

    cli: QCSClientConfigurationSettingsApplicationsCLI = Field(
        default_factory=QCSClientConfigurationSettingsApplicationsCLI
    )

    pyquil: QCSClientConfigurationSettingsApplicationsPyquil = Field(
        default_factory=QCSClientConfigurationSettingsApplicationsPyquil
    )


class QCSClientConfigurationSettingsProfile(EnvironmentModel):
    """Specifies the authorization server, credentials, and API URL.

    The attributes of this class can be used to initialize an
    ``httpx.Client`` with the correct base URL and the ``QCSAuth``
    middleware for making authenticated API calls against the QCS API.

    ``QCSClientConfigurationSettings`` may contain several profiles, which
    ``QCSClientConfiguration.profile_name`` may key into.
    """

    api_url: HttpUrl = "https://api.qcs.rigetti.com"
    """URL of the QCS API to use for all API calls"""

    auth_server_name: str = "default"
    """Key for `QCSClientConfigurationSettings.auth_servers`."""

    applications: QCSClientConfigurationSettingsApplications = Field(
        default_factory=QCSClientConfigurationSettingsApplications
    )
    """Application specific configuration values."""

    credentials_name: str = "default"
    """Key for ``QCSClientConfigurationSecrets.credentials``."""

    class Config:
        env_prefix = "QCS_SETTINGS_"


class QCSClientConfigurationSettings(QCSClientConfigurationFile):
    """A fully parsed settings configuration file.

    This contains all of the user's configured authorization servers
    and profiles. It may optionally contain a ``default_profile_name``
    to use to override the "default" value.

    ``QCSClientConfiguration`` keys into these configured values when
    instantiated.
    """

    default_profile_name: str = "default"
    """Which profile to select settings from when none is specified.

    See ``QCSClientConfiguration.load``.
    """

    profiles: Dict[str, QCSClientConfigurationSettingsProfile] = Field(
        default_factory=lambda: dict(default=QCSClientConfigurationSettingsProfile())
    )
    """All available configuration profiles, keyed by name."""

    auth_servers: Dict[str, QCSAuthServer] = Field(default_factory=lambda: {"default": _DEFAULT_AUTH_SERVER})
    """All available authorization servers, keyed by name."""



from pathlib import Path
from typing import Any, Dict, Optional

import toml
from pydantic.types import FilePath
from pydantic.utils import deep_update

from qcs_api_client.client._configuration.environment import _EnvironmentBaseModel


class QCSClientConfigurationFile(_EnvironmentBaseModel):
    """
    A configuration value store which loads field values in this order of precedence on
    initialization:
    1. Runtime keyword arguments
    2. Environment variables
    3. Configuration file path (if provided)
    4. Field default values
    """

    file_path: Optional[Path] = None
    """
    The file path which maps to this configuration, and to which this configuration will be written
    on save.
    """

    def __init__(self, **kwargs: Any) -> None:
        values = deep_update(
            self._read_file(kwargs.get("file_path")),
            self._read_env(),
            kwargs,
        )

        super().__init__(**values)

    def _read_file(self, file_path: Optional[Path]) -> Dict[str, Any]:
        """
        Read a dict from a file path if it exists.
        """
        if file_path is None:
            return {}

        path = Path(file_path)
        if not path.exists():
            return {}

        with open(path, "r") as f:
            return toml.load(f)

    def dict(self, *args, **kwargs) -> dict:
        if "exclude" in kwargs:
            kwargs["exclude"].add("file_path")
        else:
            kwargs["exclude"] = {"file_path"}

        return super().dict(*args, **kwargs)

    def json(self, *args, **kwargs) -> str:
        if "exclude" in kwargs:
            kwargs["exclude"].add("file_path")
        else:
            kwargs["exclude"] = {"file_path"}

        return super().json(*args, **kwargs)

    @classmethod
    def parse_file(cls, path: FilePath) -> "QCSClientConfigurationFile":
        with open(path, "r") as f:
            return cls(file_path=path, **toml.load(f))



import asyncio
import threading
import httpx
from pydantic import BaseModel, Field
from http import HTTPStatus
from typing import Set

from qcs_api_client.client._configuration import (
    QCSClientConfiguration,
    TokenPayload,
)


class QCSAuthConfiguration(BaseModel):
    """This configures how ``QCSAuth`` implements its access token refresh mechanism."""

    pre: bool = False
    """Pre-emptively refresh access tokens.

    When set to True, this will check the access token's expiration and refresh
    when necessary before setting the access token in the outgoing Authorization
    header.
    """

    post: bool = True
    """Refresh access tokens based on response status code.

    When set to True, this will check responses for the status codes configured
    in ``post_refresh_statuses``. On match, ``QCSAuth`` will refresh the access token
    and retry the request.
    """

    post_refresh_statuses: Set[int] = Field(default_factory=lambda: {HTTPStatus.UNAUTHORIZED, HTTPStatus.FORBIDDEN})
    """Response status codes which indicates a possible expired token payload.

    This contains a set of HTTP status codes which ``QCSAuth`` will check on
    responses when `post` is set to True.
    """


class QCSAuthRefreshError(Exception):
    def __init__(self, response: httpx.Response):
        self.response = response
        self.message = f"authentication token refresh failed with status {response.status_code}: {response.text}"
        super().__init__(self.message)


class QCSAuth(httpx.Auth):
    """Implements ``httpx.Auth`` ``sync_auth_flow`` and ``async_auth_flow``.

    If the ``QCSClientConfiguration`` that initializes this class has a valid
    ``TokenPayload`` on ``QCSClientConfiguration.credentials``, it will set the
    a refreshed access token as a Bearer token on the Authorization header
    of outgoing requests.

    Access tokens are refreshed via OAuth2 refresh mechanism as indicated
    by ``QCSAuthConfiguration``.
    """

    def __init__(self, client_configuration: QCSClientConfiguration, auth_configuration: QCSAuthConfiguration = None):
        self._client_configuration = client_configuration
        self._sync_lock = threading.RLock()
        self._async_lock = asyncio.Lock()
        self._auth_configuration = auth_configuration or QCSAuthConfiguration()

    def sync_refresh_token(self):
        with self._sync_lock:
            refresh_token = self._client_configuration.credentials.refresh_token
            res = _do_refresh_token(
                self._client_configuration.auth_server.token_url(),
                self._client_configuration.auth_server.client_id,
                refresh_token,
            )
            if res.status_code != HTTPStatus.OK:
                raise QCSAuthRefreshError(response=res)
            token_payload = TokenPayload(**res.json())
            self._client_configuration.secrets.update_token(
                credentials_name=self._client_configuration.profile_name, token=token_payload
            )

    def sync_auth_flow(self, request):
        if self._client_configuration.credentials.token_payload is None:
            yield request
            return

        if self._auth_configuration.pre and self._client_configuration.credentials.token_payload.should_refresh():
            self.sync_refresh_token()

        request.headers["Authorization"] = f"Bearer {self._client_configuration.credentials.access_token}"
        response = yield request

        if self._auth_configuration.post and response.status_code in self._auth_configuration.post_refresh_statuses:
            self.sync_refresh_token()
            request.headers["Authorization"] = f"Bearer {self._client_configuration.credentials.access_token}"
            yield request

    async def async_refresh_token(self):
        async with self._async_lock:
            refresh_token = self._client_configuration.credentials.refresh_token
            res = await _do_refresh_token_async(
                self._client_configuration.auth_server.token_url(),
                self._client_configuration.auth_server.client_id,
                refresh_token,
            )
            if res.status_code != HTTPStatus.OK:
                raise QCSAuthRefreshError(response=res)
            token_payload = TokenPayload(**res.json())
            self._client_configuration.secrets.update_token(
                credentials_name=self._client_configuration.profile_name, token=token_payload
            )

    async def async_auth_flow(self, request):
        if self._client_configuration.credentials.token_payload is None:
            yield request
            return

        if self._auth_configuration.pre and self._client_configuration.credentials.token_payload.should_refresh():
            await self.async_refresh_token()

        request.headers["Authorization"] = f"Bearer {self._client_configuration.credentials.access_token}"
        response = yield request

        if self._auth_configuration.post and response.status_code in self._auth_configuration.post_refresh_statuses:
            await self.async_refresh_token()
            request.headers["Authorization"] = f"Bearer {self._client_configuration.credentials.access_token}"
            yield request


def _do_refresh_token(token_url: str, client_id: str, refresh_token: str):
    data = {"grant_type": "refresh_token", "client_id": client_id, "refresh_token": refresh_token}
    return httpx.request(
        "POST",
        token_url,
        data=data,
    )


async def _do_refresh_token_async(token_url: str, client_id: str, refresh_token: str):
    data = {"grant_type": "refresh_token", "client_id": client_id, "refresh_token": refresh_token}
    async with httpx.AsyncClient() as client:
        return await client.request(
            "POST",
            token_url,
            data=data,
        )



from qcs_api_client.client._configuration.configuration import QCSClientConfiguration
from qcs_api_client.client._configuration.error import QCSClientConfigurationError
from qcs_api_client.client._configuration.secrets import (
    QCSClientConfigurationSecrets,
    QCSClientConfigurationSecretsCredentials,
    TokenPayload,
)
from qcs_api_client.client._configuration.settings import (
    QCSAuthServer,
    QCSClientConfigurationSettings,
    QCSClientConfigurationSettingsProfile,
    QCSClientConfigurationSettingsApplications,
    QCSClientConfigurationSettingsApplicationsCLI,
)



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..models.engagement_credentials import EngagementCredentials
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class EngagementWithCredentials:
    """ An engagement is the authorization of a user to execute work on a Quantum Processor Endpoint. """

    address: str
    credentials: EngagementCredentials
    endpoint_id: str
    expires_at: str
    quantum_processor_id: str
    user_id: str
    minimum_priority: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        address = self.address
        credentials = self.credentials.to_dict()

        endpoint_id = self.endpoint_id
        expires_at = self.expires_at
        quantum_processor_id = self.quantum_processor_id
        user_id = self.user_id
        minimum_priority = self.minimum_priority

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "address": address,
                "credentials": credentials,
                "endpointId": endpoint_id,
                "expiresAt": expires_at,
                "quantumProcessorId": quantum_processor_id,
                "userId": user_id,
            }
        )
        if minimum_priority is not UNSET:
            field_dict["minimumPriority"] = minimum_priority

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "EngagementWithCredentials":
        d = src_dict.copy()
        address = d.pop("address")

        credentials = EngagementCredentials.from_dict(d.pop("credentials"))

        endpoint_id = d.pop("endpointId")

        expires_at = d.pop("expiresAt")

        quantum_processor_id = d.pop("quantumProcessorId")

        user_id = d.pop("userId")

        minimum_priority = d.pop("minimumPriority", UNSET)

        engagement_with_credentials = EngagementWithCredentials(
            address=address,
            credentials=credentials,
            endpoint_id=endpoint_id,
            expires_at=expires_at,
            quantum_processor_id=quantum_processor_id,
            user_id=user_id,
            minimum_priority=minimum_priority,
        )

        engagement_with_credentials.additional_properties = d
        return engagement_with_credentials

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .add_group_user_request import AddGroupUserRequest
from .architecture import Architecture
from .auth_email_password_reset_token_request import AuthEmailPasswordResetTokenRequest
from .auth_reset_password_request import AuthResetPasswordRequest
from .auth_reset_password_with_token_request import AuthResetPasswordWithTokenRequest
from .available_reservation import AvailableReservation
from .characteristic import Characteristic
from .check_client_application_request import CheckClientApplicationRequest
from .check_client_application_response import CheckClientApplicationResponse
from .checksum_description import ChecksumDescription
from .checksum_description_type import ChecksumDescriptionType
from .client_application import ClientApplication
from .client_applications_download_link import ClientApplicationsDownloadLink
from .create_endpoint_parameters import CreateEndpointParameters
from .create_engagement_request import CreateEngagementRequest
from .create_reservation_request import CreateReservationRequest
from .dictionary import Dictionary
from .edge import Edge
from .endpoint import Endpoint
from .endpoint_addresses import EndpointAddresses
from .engagement_credentials import EngagementCredentials
from .engagement_with_credentials import EngagementWithCredentials
from .error import Error
from .family import Family
from .filter import Filter
from .find_available_reservations_response import FindAvailableReservationsResponse
from .get_quilt_calibrations_response import GetQuiltCalibrationsResponse
from .group import Group
from .health import Health
from .instruction_set_architecture import InstructionSetArchitecture
from .internal_create_reservation_request import InternalCreateReservationRequest
from .invite_user_request import InviteUserRequest
from .list_client_applications_response import ListClientApplicationsResponse
from .list_endpoints_response import ListEndpointsResponse
from .list_group_users_response import ListGroupUsersResponse
from .list_groups_response import ListGroupsResponse
from .list_quantum_processors_response import ListQuantumProcessorsResponse
from .list_reservations_response import ListReservationsResponse
from .node import Node
from .operation import Operation
from .operation_site import OperationSite
from .order import Order
from .parameter import Parameter
from .parameter_spec import ParameterSpec
from .quantum_processor import QuantumProcessor
from .remove_group_user_request import RemoveGroupUserRequest
from .reservation import Reservation
from .translate_native_quil_to_encrypted_binary_request import TranslateNativeQuilToEncryptedBinaryRequest
from .translate_native_quil_to_encrypted_binary_response import TranslateNativeQuilToEncryptedBinaryResponse
from .translate_native_quil_to_encrypted_binary_response_memory_descriptors import (
    TranslateNativeQuilToEncryptedBinaryResponseMemoryDescriptors,
)
from .user import User
from .user_credentials import UserCredentials
from .user_credentials_password import UserCredentialsPassword
from .user_profile import UserProfile
from .validation_error import ValidationError
from .validation_error_in import ValidationErrorIn



import datetime
from typing import Any, Callable, Dict, List, Optional, Union, cast

import attr
from dateutil.parser import isoparse
from rfc3339 import rfc3339

from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Reservation:
    """  """

    created_time: datetime.datetime
    end_time: datetime.datetime
    id: int
    price: int
    quantum_processor_id: str
    start_time: datetime.datetime
    user_id: str
    cancelled: Union[Unset, bool] = UNSET
    notes: Union[Unset, str] = UNSET
    updated_time: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        assert self.created_time.tzinfo is not None, "Datetime must have timezone information"
        created_time = rfc3339(self.created_time)

        assert self.end_time.tzinfo is not None, "Datetime must have timezone information"
        end_time = rfc3339(self.end_time)

        id = self.id
        price = self.price
        quantum_processor_id = self.quantum_processor_id
        assert self.start_time.tzinfo is not None, "Datetime must have timezone information"
        start_time = rfc3339(self.start_time)

        user_id = self.user_id
        cancelled = self.cancelled
        notes = self.notes
        updated_time: Union[Unset, str] = UNSET
        if not isinstance(self.updated_time, Unset):
            assert self.updated_time.tzinfo is not None, "Datetime must have timezone information"
            updated_time = rfc3339(self.updated_time)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdTime": created_time,
                "endTime": end_time,
                "id": id,
                "price": price,
                "quantumProcessorId": quantum_processor_id,
                "startTime": start_time,
                "userId": user_id,
            }
        )
        if cancelled is not UNSET:
            field_dict["cancelled"] = cancelled
        if notes is not UNSET:
            field_dict["notes"] = notes
        if updated_time is not UNSET:
            field_dict["updatedTime"] = updated_time

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Reservation":
        d = src_dict.copy()
        created_time = isoparse(d.pop("createdTime"))

        end_time = isoparse(d.pop("endTime"))

        id = d.pop("id")

        price = d.pop("price")

        quantum_processor_id = d.pop("quantumProcessorId")

        start_time = isoparse(d.pop("startTime"))

        user_id = d.pop("userId")

        cancelled = d.pop("cancelled", UNSET)

        notes = d.pop("notes", UNSET)

        updated_time = None
        updated_time__input = d.pop("updatedTime", UNSET)
        if isinstance(updated_time__input, str):
            updated_time = isoparse(cast(str, updated_time__input))

        reservation = Reservation(
            created_time=created_time,
            end_time=end_time,
            id=id,
            price=price,
            quantum_processor_id=quantum_processor_id,
            start_time=start_time,
            user_id=user_id,
            cancelled=cancelled,
            notes=notes,
            updated_time=updated_time,
        )

        reservation.additional_properties = d
        return reservation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Callable, Dict, List, Optional, Union, cast

import attr
from dateutil.parser import isoparse
from rfc3339 import rfc3339

from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Characteristic:
    """ A measured characteristic of an operation. """

    name: str
    timestamp: datetime.datetime
    value: float
    error: Union[Unset, float] = UNSET
    node_ids: Union[Unset, List[int]] = UNSET
    parameter_values: Union[Unset, List[float]] = UNSET

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        name = self.name
        assert self.timestamp.tzinfo is not None, "Datetime must have timezone information"
        timestamp = rfc3339(self.timestamp)

        value = self.value
        error = self.error
        node_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.node_ids, Unset):
            node_ids = self.node_ids

        parameter_values: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.parameter_values, Unset):
            parameter_values = self.parameter_values

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "timestamp": timestamp,
                "value": value,
            }
        )
        if error is not UNSET:
            field_dict["error"] = error
        if node_ids is not UNSET:
            field_dict["node_ids"] = node_ids
        if parameter_values is not UNSET:
            field_dict["parameter_values"] = parameter_values

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Characteristic":
        d = src_dict.copy()
        name = d.pop("name")

        timestamp = isoparse(d.pop("timestamp"))

        value = d.pop("value")

        error = d.pop("error", UNSET)

        node_ids = cast(List[int], d.pop("node_ids", UNSET))

        parameter_values = cast(List[float], d.pop("parameter_values", UNSET))

        characteristic = Characteristic(
            name=name,
            timestamp=timestamp,
            value=value,
            error=error,
            node_ids=node_ids,
            parameter_values=parameter_values,
        )

        return characteristic



from typing import Any, Callable, Dict, List, Optional

import attr

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class AuthResetPasswordRequest:
    """  """

    new_password: str
    old_password: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        new_password = self.new_password
        old_password = self.old_password

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "newPassword": new_password,
                "oldPassword": old_password,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "AuthResetPasswordRequest":
        d = src_dict.copy()
        new_password = d.pop("newPassword")

        old_password = d.pop("oldPassword")

        auth_reset_password_request = AuthResetPasswordRequest(
            new_password=new_password,
            old_password=old_password,
        )

        auth_reset_password_request.additional_properties = d
        return auth_reset_password_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Callable, Dict, List, Optional

import attr

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Filter:
    """A string conforming to a *limited* set of the filtering operations described in [Google AIP 160](https://google.aip.dev/160).

    * Expressions are always of the form `{field} {operator} {value}` and may be grouped with `()` and joined with `AND` or `OR`.
    * Fields are specific to the route in question, but are typically a subset of attributes of the requested resource.
    * Operators are limited to `=`, `>`, `>=`, `<`, `<=`, and `!=`.
    * Values may take the following forms:
      * `true` or `false` for boolean fields
      * a number
      * a string (include surrounding `"`s),
      * a duration string (include surrounding `"`s). Valid time units are "ns", "us" (or "µs"), "ms", "s", "m", "h".
      * a date string (include surrounding `"`s). Should be formatted [RFC3339 5.6](https://tools.ietf.org/html/rfc3339#section-5.6).

    For example, `startTime >= "2020-06-24T22:00:00.000Z" OR (duration >= "15m" AND endTime < "2020-06-24T22:00:00.000Z")`.
    """

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Filter":
        d = src_dict.copy()
        filter = Filter()

        filter.additional_properties = d
        return filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains some shared types for properties """
from typing import BinaryIO, Callable, Generic, Optional, TextIO, Tuple, TypeVar, Union

import attr
import httpx


class Unset:
    def __bool__(self) -> bool:
        return False


UNSET: Unset = Unset()


@attr.s(auto_attribs=True)
class File:
    """ Contains information for file uploads """

    payload: Union[BinaryIO, TextIO]
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> Tuple[Optional[str], Union[BinaryIO, TextIO], Optional[str]]:
        """ Return a tuple representation that httpx will accept for multipart/form-data """
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


class Response(httpx.Response, Generic[T]):
    """
    Response from an API endpoint.

    Serves as a minimal wrapper around an httpx.Response object, supporting parsing into a
    known API response type.
    """

    _parsed: Optional[T]
    """The response body parsed into an API model instance."""

    _parse_function: Callable[[httpx.Response], T]
    """The function responsible for parsing the response body into an API model instance."""

    @classmethod
    def build_from_httpx_response(
        cls, *, response: httpx.Response, parse_function: Callable[[httpx.Response], T]
    ) -> "Response":
        """
        Mutate an ``httpx.Response`` into a ``Response`` by respecifying the class and applying
        required values.
        """
        response.__class__ = cls
        response._parsed = None
        response._parse_function = parse_function
        return response

    @property
    def parsed(self) -> T:
        """
        Return the response body parsed into an API model.

        Value is memoized after the first successful call.
        """
        if self._parsed is None:
            self._parsed = self._parse_function(response=self)

        return self._parsed


__all__ = ["File", "Response"]



from typing import Any, Callable, Dict, List, Optional, Union, cast

import attr

from ..models.validation_error_in import ValidationErrorIn
from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class ValidationError:
    """  """

    in_: ValidationErrorIn
    message: str
    path: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        in_ = self.in_.value

        message = self.message
        path: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.path, Unset):
            path = self.path

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "in": in_,
                "message": message,
            }
        )
        if path is not UNSET:
            field_dict["path"] = path

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "ValidationError":
        d = src_dict.copy()
        in_ = ValidationErrorIn(d.pop("in"))

        message = d.pop("message")

        path = cast(List[str], d.pop("path", UNSET))

        validation_error = ValidationError(
            in_=in_,
            message=message,
            path=path,
        )

        validation_error.additional_properties = d
        return validation_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import os
from pathlib import Path
from typing import Optional

from pydantic.main import BaseModel

from qcs_api_client.client._configuration.error import QCSClientConfigurationError
from qcs_api_client.client._configuration.secrets import (
    QCSClientConfigurationSecrets,
    QCSClientConfigurationSecretsCredentials,
)
from qcs_api_client.client._configuration.settings import (
    QCSClientConfigurationSettings,
    QCSClientConfigurationSettingsProfile,
)


QCS_BASE_PATH = Path("~/.qcs").expanduser()

DEFAULT_SECRETS_FILE_PATH = QCS_BASE_PATH / "secrets.toml"
DEFAULT_SETTINGS_FILE_PATH = QCS_BASE_PATH / "settings.toml"


class QCSClientConfiguration(BaseModel):
    """A user's settings and secrets along with a specified profile name.

    This class contains a full representation of user specified
    ``QCSClientConfigurationSecrets`` and ``QCSClientConfigurationSettings``, as
    well as a ``profile_name`` which indicates which
    ``QCSClientConfigurationSettingsProfile`` to access within
    ``QCSClientConfigurationSettings.profiles``.

    Typically, clients will simply call ``QCSClientConfiguration.load``,
    to initialize this class from the specified secrets and settings
    paths.
    """

    profile_name: str
    secrets: QCSClientConfigurationSecrets
    settings: QCSClientConfigurationSettings

    @property
    def auth_server(self):
        """Returns the configured authorization server.

        ``self.profile.auth_server_name`` serves as key to
        ``QCSClientConfigurationSettings.auth_servers``.

        Returns:
            The specified ``QCSAuthServer``.

        Raises:
             QCSClientConfigurationError: If
                ``QCSClientConfigurationSettings.auth_servers`` does not have
                a value for the authorization server name.
        """
        server = self.settings.auth_servers.get(self.profile.auth_server_name)
        if server is None:
            raise QCSClientConfigurationError(f"no authorization server configured for {self.profile.auth_server_name}")

        return server

    @property
    def credentials(self) -> QCSClientConfigurationSecretsCredentials:
        """Returns the configured ``QCSClientConfigurationSecretsCredentials``

        ``self.profile.credentials_name`` serves as key to
        ``QCSClientConfigurationSecrets.credentials``.

        Returns:
            The specified ``QCSClientConfigurationSecretsCredentials``.

        Raises:
            QCSClientConfigurationError: If
                ``QCSClientConfigurationSettings.credentials`` does not have
                a value for the specified credentials name.
        """
        credentials = self.secrets.credentials.get(self.profile.credentials_name)
        if credentials is None:
            raise QCSClientConfigurationError(f"no credentials available named '{self.profile.credentials_name}'")
        return credentials

    @property
    def profile(self) -> QCSClientConfigurationSettingsProfile:
        """Returns the configured ``QCSClientConfigurationSettingsProfile``.

        `self.profile_name` serves as key to
        ``QCSClientConfigurationSettingsProfile.profiles``.

        Returns:
            The specified ``QCSClientConfigurationSettingsProfile``.

        Raises:
            QCSClientConfigurationError: If
                ``QCSClientConfigurationSettings.profiles`` does not have
                a value for the specified profile name.
        """
        profile = self.settings.profiles.get(self.profile_name)
        if profile is None:
            raise QCSClientConfigurationError(f"no profile available named '{self.profile_name}'")
        return profile

    @classmethod
    def load(
        cls,
        profile_name: Optional[str] = None,
        settings_file_path: Optional[os.PathLike] = None,
        secrets_file_path: Optional[os.PathLike] = None,
    ) -> "QCSClientConfiguration":
        """Loads a fully specified ``QCSClientConfiguration`` from file.

        It evaluates attribute values according to the following precedence:
        argument value > environment variable > default value.

        Args:
            profile_name: [env: QCS_PROFILE_NAME] The name of the profile
                referenced in the fully parsed ``QCSClientConfigurationSettings.profiles``.
                If the profile name does not exist on ``QCSClientConfigurationSettings``,
                ``QCSClientConfiguration.profile`` will raise an error.
                The default value is "default", which may be overridden by
                ``QCSClientConfigurationSettings.default_profile_name``.
            settings_file_path: [env: QCS_SETTINGS_FILE_PATH] The file path
                from which to parse ``QCSClientConfigurationSettings``. This file
                must exist in TOML format. The default value is ``~/.qcs/settings.toml``.
            secrets_file_path: [env: QCS_SECRETS_FILE_PATH] The file path
                from which to parse ``QCSClientConfigurationSecrets``. This file
                must exist in TOML format. The default value is ``~/.qcs/secrets.toml``.

        Returns:
            A fully specified ``QCSClientConfiguration``, which ``QCSAuth`` may use for
            adding refreshed OAuth2 access tokens to outgoing HTTP requests.
        """
        secrets_file_path = secrets_file_path or os.getenv("QCS_SECRETS_FILE_PATH", DEFAULT_SECRETS_FILE_PATH)

        secrets = QCSClientConfigurationSecrets(file_path=secrets_file_path)

        settings_file_path = settings_file_path or os.getenv("QCS_SETTINGS_FILE_PATH", DEFAULT_SETTINGS_FILE_PATH)

        settings = QCSClientConfigurationSettings(file_path=settings_file_path)

        profile_name = profile_name or os.getenv("QCS_PROFILE_NAME", settings.default_profile_name)

        return cls(profile_name=profile_name, secrets=secrets, settings=settings)



from typing import Any, Callable, Dict, List, Optional, Union

import attr

from ..types import UNSET, Unset
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class TranslateNativeQuilToEncryptedBinaryRequest:
    """  """

    num_shots: int
    quil: str
    settings_timestamp: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        num_shots = self.num_shots
        quil = self.quil
        settings_timestamp = self.settings_timestamp

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "numShots": num_shots,
                "quil": quil,
            }
        )
        if settings_timestamp is not UNSET:
            field_dict["settingsTimestamp"] = settings_timestamp

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "TranslateNativeQuilToEncryptedBinaryRequest":
        d = src_dict.copy()
        num_shots = d.pop("numShots")

        quil = d.pop("quil")

        settings_timestamp = d.pop("settingsTimestamp", UNSET)

        translate_native_quil_to_encrypted_binary_request = TranslateNativeQuilToEncryptedBinaryRequest(
            num_shots=num_shots,
            quil=quil,
            settings_timestamp=settings_timestamp,
        )

        translate_native_quil_to_encrypted_binary_request.additional_properties = d
        return translate_native_quil_to_encrypted_binary_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qcs_api_client',
 'qcs_api_client.api',
 'qcs_api_client.api.account',
 'qcs_api_client.api.authentication',
 'qcs_api_client.api.client_applications',
 'qcs_api_client.api.default',
 'qcs_api_client.api.endpoints',
 'qcs_api_client.api.engagements',
 'qcs_api_client.api.quantum_processors',
 'qcs_api_client.api.reservations',
 'qcs_api_client.api.translation',
 'qcs_api_client.client',
 'qcs_api_client.client._configuration',
 'qcs_api_client.models',
 'qcs_api_client.operations',
 'qcs_api_client.operations.asyncio',
 'qcs_api_client.operations.asyncio_from_dict',
 'qcs_api_client.operations.sync',
 'qcs_api_client.operations.sync_from_dict',
 'qcs_api_client.util']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<21.0.0',
 'httpx>=0.15.0,<0.16.0',
 'iso8601>=0.1.13,<0.2.0',
 'pydantic>=1.7.2,<2.0.0',
 'pyjwt>=1.7.1,<2.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'retrying>=1.3.3,<2.0.0',
 'rfc3339>=6.2,<7.0',
 'toml>=0.10.2,<0.11.0']

setup_kwargs = {
    'name': 'qcs-api-client',
    'version': '0.20.0',
    'description': 'A client library for accessing the Rigetti QCS API',
    'long_description': '[![Documentation Status](https://readthedocs.org/projects/qcs-api-client-python/badge/?version=latest)](https://qcs-api-client-python.readthedocs.io/en/latest/?badge=latest)\n\n# QCS API Client\n\nA client library for accessing the [Rigetti QCS API](https://docs.api.qcs.rigetti.com/).\n\n## Usage\n\n### Synchronous Usage\n\n```python\nfrom qcs_api_client.client import build_sync_client\nfrom qcs_api_client.models import ListReservationsResponse\nfrom qcs_api_client.operations.sync import list_reservations\n\nwith build_sync_client() as client:\n    response: ListReservationsResponse = list_reservations(client=client).parsed\n```\n\n### Asynchronous Usage\n\n```python\nfrom qcs_api_client.client import build_async_client\nfrom qcs_api_client.models import ListReservationsResponse\nfrom qcs_api_client.operations.asyncio import list_reservations\n\n# Within an event loop:\nasync with build_async_client() as client:\n    response: ListReservationsResponse = await list_reservations(client=client).parsed\n```\n\n### Configuration\n\nBy default, initializing your client with `build_sync_client` or `build_async_client` will\nuse `QCSClientConfiguation.load` to load default configuration values. This function accepts:\n\n- A profile name (env: `QCS_PROFILE_NAME`). The name of the profile referenced in your settings\n  file. If not provided, `QCSClientConfiguation.load` will evaluate this to a `default_profile_name`\n  set in your settings file or "default".\n- A settings file path (env: `QCS_SETTINGS_FILE_PATH`). A path to the current user\'s settings file in TOML format. If not provided,  `QCSClientConfiguation.load` will evaluate this to `~/.qcs/settings.toml`.\n- A secrets file path (env: `QCS_SECRETS_FILE_PATH`). A path to the current user\'s secrets file in TOML format. If not provided,  `QCSClientConfiguation.load` will evaluate this to `~/.qcs/secrets.toml`. The user should have write access to this file, as the client will attempt to update the file with refreshed access tokens as necessary.\n     \nIf you need to specify a custom profile name or path you can initialize your client accordingly:\n\n```python\nfrom qcs_api_client.client import build_sync_client, QCSClientConfiguration\nfrom qcs_api_client.models import ListReservationsResponse\nfrom qcs_api_client.operations.sync import list_reservations\n\nconfiguration = QCSClientConfiguration.load(\n    profile_name=\'custom\',\n    secrets_file_path=\'./path/to/custom/secrets.toml\',\n    settings_file_path=\'./path/to/custom/settings.toml\',\n)\n\nwith build_sync_client(configuration=configuration) as client:\n    response: ListReservationsResponse = list_reservations(client=client).parsed\n```\n\n## Development\n\nThe source code for this repository is synchronized from another source. No commits made directly to GitHub will be retained.\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rigetti/qcs-api-client-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Callable, Dict, Optional

import attr

from ..types import UNSET
from ..util.serialization import is_not_none


@attr.s(auto_attribs=True)
class Parameter:
    """ A parameter to an operation. """

    name: str

    def to_dict(self, pick_by_predicate: Optional[Callable[[Any], bool]] = is_not_none) -> Dict[str, Any]:
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )

        field_dict = {k: v for k, v in field_dict.items() if v != UNSET}
        if pick_by_predicate is not None:
            field_dict = {k: v for k, v in field_dict.items() if pick_by_predicate(v)}

        return field_dict

    @staticmethod
    def from_dict(src_dict: Dict[str, Any]) -> "Parameter":
        d = src_dict.copy()
        name = d.pop("name")

        parameter = Parameter(
            name=name,
        )

        return parameter
